IMPLEMENTATION MODULE RSTACKELEMENT;

IMPORT ITEMS;

FROM ERRORHANDLER 
   IMPORT ErrorHandler;

FROM STACKELEMENTSEXCEPTIONS
   IMPORT Exit, Warning;


CONST ModuleID = "RSTACKELEMENT";

TYPE StackElement = ITEMS.Item;

(*
     Operators = (add,sub,mul,div,res,undefined);
     Numbers   = (integers,reals);
*)

     ElementType = (Operand, Operator);

     Operands = RECORD
        CASE Num : Numbers OF
           integers : IntNum : INTEGER
        |  reals    : RealNum : REAL
        END (* CASE *)
     END (* Operands *);

     Element = RECORD
        CASE El : ElementType OF
           Operator : Opr : Operators
        |  Operand  : Opd : Operands
        END (* CASE *)
     END (* Element *);


PROCEDURE CreateInteger (n : INTEGER) : StackElement;
VAR E : Element;
    S : StackElement;
BEGIN
   WITH E DO
      El := Operand;
      WITH Opd DO
         Num := integers;
         IntNum := n
      END (* WITH *)
   END (* WITH *);
   S := ITEMS.Create (E);
   RETURN S
END CreateInteger;


PROCEDURE CreateReal (n : REAL) : StackElement;
VAR E : Element;
    S : StackElement;
BEGIN
   WITH E DO
      El := Operand;
      WITH Opd DO
         Num := reals;
         RealNum := n
      END (* WITH *)
   END (* WITH *);
   S := ITEMS.Create (E);
   RETURN S
END CreateReal;


PROCEDURE CreateOperator (o : CHAR) : StackElement;
CONST ProcID = "CreateOperator";
VAR E : Element;
    S : StackElement;
BEGIN
   WITH E DO
      El := Operator;
      CASE o OF
         "+" : Opr := add
      |  "-" : Opr := sub
      |  "*" : Opr := mul
      |  "/" : Opr := div
      |  "=" : Opr := res
      ELSE Opr := undefined;
           ErrorHandler (ModuleID,ProcID,4,Warning);
      END (* CASE *)
   END (* WITH *);
   S := ITEMS.Create (E);
   RETURN S
END CreateOperator;


PROCEDURE IsInteger (s : StackElement) : BOOLEAN;
CONST ProcID = "IsInteger";
VAR E : Element;
BEGIN
   ITEMS.GetItem (E,s);
   IF NOT IsOperator (s) THEN
      RETURN E.Opd.Num = integers
   ELSE
      (* ErrorHandler (ModuleID,ProcID,2,Warning); *)
      RETURN FALSE
   END (* IF *)
END IsInteger;

   
PROCEDURE IsReal (s : StackElement) : BOOLEAN;
CONST ProcID = "IsReal";
VAR E : Element;
BEGIN
   ITEMS.GetItem (E,s);
   IF NOT IsOperator (s) THEN
      RETURN E.Opd.Num = reals
   ELSE
      (* ErrorHandler (ModuleID,ProcID,2,Warning); *)
      RETURN FALSE
   END (* IF *)
END IsReal;


PROCEDURE IsOperator (s : StackElement) : BOOLEAN;
CONST ProcID = "IsOperator";
VAR E : Element;
BEGIN
   ITEMS.GetItem (E,s);
   IF  E.El # Operator THEN
      (* ErrorHandler (ModuleID,ProcID,3,Warning); *)
      RETURN FALSE
   ELSE
      RETURN TRUE
   END (* IF *)
END IsOperator;


PROCEDURE GetNumberType (s : StackElement) : Numbers;
CONST ProcID = "GetNumberType";
BEGIN
   IF IsInteger (s) THEN
      RETURN integers
   ELSIF IsReal (s) THEN
      RETURN reals
   ELSE
      (* ErrorHandler (ModuleID,ProcID,5,Warning); *)
      RETURN notype
   END (* IF *)
END GetNumberType;


PROCEDURE GetInteger (s : StackElement) : INTEGER;
CONST ProcID = "GetInteger";
VAR E : Element;
BEGIN
   IF NOT IsOperator (s) THEN
      ITEMS.GetItem (E,s);
      RETURN E.Opd.IntNum
   ELSE
      ErrorHandler (ModuleID,ProcID,1,Exit)
   END (* IF *)
END GetInteger;


PROCEDURE GetReal (s : StackElement) : REAL;
CONST ProcID = "GetReal";
VAR E : Element;
BEGIN
   IF NOT IsOperator (s) THEN
      ITEMS.GetItem (E,s);
      RETURN E.Opd.RealNum
   ELSE
      ErrorHandler (ModuleID,ProcID,1,Exit)
   END (* IF *)
END GetReal;


PROCEDURE GetOperator (s : StackElement) : Operators;
CONST ProcID = "GetOperator";
VAR E : Element;
BEGIN
   IF IsOperator (s) THEN
      ITEMS.GetItem (E,s);
      RETURN E.Opr
   ELSE
      (* ErrorHandler (ModuleID,ProcID,4,Warning); *)
      RETURN undefined
   END (* IF *)
END GetOperator;


END RSTACKELEMENT.
